#######
#Import of libraries#
import numpy as np
import cv2
import datetime
from time import sleep
from sense_hat import SenseHat

#######
#Display Setup#

sense = SenseHat()
sense.clear()

#######
#Data Collection Setup#

with open ("HumanLifeInSpaceResults.csv", "w") as file:
    file.write("Time , Temperature ,Pressure ,Humidity, Brightness, MotionDetection \n")

########
#Motion Detection Setup#
#(Some aspects of the Motion Detection code have been used from an open source project by Intel, which has later been removed)#
sdThresh = 10
#TODO: Face Detection 1

def distMap(frame1, frame2):
    """outputs pythagorean distance between two frames"""
    frame1_32 = np.float32(frame1)
    frame2_32 = np.float32(frame2)
    diff32 = frame1_32 - frame2_32
    norm32 = np.sqrt(diff32[:,:,0]**2 + diff32[:,:,1]**2 + diff32[:,:,2]**2)/np.sqrt(255**2 + 255**2 + 255**2)
    dist = np.uint8(norm32*255)
    return dist

#capture video stream from camera source. 0 refers to first camera, 1 referes to 2nd and so on.
cap = cv2.VideoCapture(0)

_, frame1 = cap.read()
_, frame2 = cap.read()

facecount = 0


while(True):
    ######################################################
    #Motion Detection#
    #(Some aspects of the Motion Detection code have been used from an open source project by Intel, which has later been removed)#
    _, frame3 = cap.read()
    rows, cols, _ = np.shape(frame3)
    dist = distMap(frame1, frame3)

    frame1 = frame2
    frame2 = frame3
    
    ######################################################
    #Brightness Detection#
    
    # here we convert the frame to the HSV colour format, where the 'v' value describes the brightness value.
    light = cv2.cvtColor(frame1, cv2.COLOR_BGR2HSV)
    h, s, v = cv2.split(light)
    brightness = np.mean(v)
    
    ######################################################
    
    # apply Gaussian smoothing which makes deviation detection easier
    mod = cv2.GaussianBlur(dist, (9,9), 0)
    
    # apply thresholding
    _, thresh = cv2.threshold(mod, 100, 255, 0)

    # calculate st dev test
    _, stDev = cv2.meanStdDev(mod)
    
    
    ######################################################
    #Time, Temp, Humi, Press#
    
    t = sense.get_temperature()
    t = round(t,1)
    t = t - 12
    # we make sure to readjust our temperature measurements to take into account that our temperature sensor is situated close to the hot chipset
    p = sense.get_pressure()
    h = sense.get_humidity()
    now_time = datetime.datetime.now()
    
    ######################################################
    #Data Collection#
    
    # Here we represent our motion detection in binary values (as a boolean), where a value of 0 means that no motion has been detected, and a value of 1 means that motion has been detected
    if stDev > sdThresh:
        with open ("HumanLifeInSpaceResults.csv", "a") as file:
            file.write("%s, %s, %s, %s, %s, 1\n" % (now_time, t, p, h, brightness))
    else:
        with open ("HumanLifeInSpaceResults.csv", "a") as file:
            file.write("%s, %s, %s, %s, %s, 0 \n" % (now_time, t, p, h, brightness))
        
    ######################################################
    #Display Metrics#
    
    # for our display metrics, we represent our data in bargraphs
        
    #Temperature#
    if 0 < t < 100:
        sense.set_pixel(0,7,255,0,0)
    else:
        sense.set_pixel(0,7,0,0,0)
    if 15 < t < 100:
        sense.set_pixel(0,6,255,0,0)
    else:
        sense.set_pixel(0,6,0,0,0)
    if 20 < t < 100:
        sense.set_pixel(0,5,255,0,0)
    else:
        sense.set_pixel(0,5,0,0,0)
    if 25 < t < 100:
        sense.set_pixel(0,4,255,0,0)
    else:
        sense.set_pixel(0,4,0,0,0)
    if 30 < t < 100:
        sense.set_pixel(0,3,255,0,0)
    else:
        sense.set_pixel(0,3,0,0,0)
        
    #Pressure#
    if 950 < p < 10000:
        sense.set_pixel(1,7,255,255,0)
    else:
        sense.set_pixel(1,7,0,0,0)
    if 975 < p < 10000:
        sense.set_pixel(1,6,255,255,0)
    else:
        sense.set_pixel(1,6,0,0,0)
    if 1000 < p < 10000:
        sense.set_pixel(1,5,255,255,0)
    else:
        sense.set_pixel(1,5,0,0,0)
    if 1025 < p < 10000:
        sense.set_pixel(1,4,255,255,0)
    else:
        sense.set_pixel(1,4,0,0,0)
    if 1050 < p < 10000:
        sense.set_pixel(1,3,255,255,0)
    else:
        sense.set_pixel(1,3,0,0,0)
    
    #Humidity#
    if 25 < h < 100:
        sense.set_pixel(2,7,0,0,255)
    else:
        sense.set_pixel(2,7,0,0,0)
    if 35 < h < 100:
        sense.set_pixel(2,6,0,0,255)
    else:
        sense.set_pixel(2,6,0,0,0)  
    if 45 < h < 100:
        sense.set_pixel(2,5,0,0,255)
    else:
        sense.set_pixel(2,5,0,0,0)
    if 55 < h < 100:
        sense.set_pixel(2,4,0,0,255)
    else:
        sense.set_pixel(2,4,0,0,0)
    if 65 < h < 100:
        sense.set_pixel(2,3,0,0,255)
    else:
        sense.set_pixel(2,3,0,0,0)
    if 75 < h < 100:
        sense.set_pixel(2,2,0,0,255)
    else:
        sense.set_pixel(2,2,0,0,0)
        
    #Brightness#
    if 50 < brightness < 255:
        sense.set_pixel(3,7,255,255,255)
    else:
        sense.set_pixel(3,7,0,0,0)
    if 75 < brightness < 255:
        sense.set_pixel(3,6,255,255,255)
    else:
        sense.set_pixel(3,6,0,0,0)
    if 100 < brightness < 255:
        sense.set_pixel(3,5,255,255,255)
    else:
        sense.set_pixel(3,5,0,0,0)
    if 125 < brightness < 255:
        sense.set_pixel(3,4,255,255,255)
    else:
        sense.set_pixel(3,4,0,0,0)
    if 150 < brightness < 255:
        sense.set_pixel(3,3,255,255,255)
    else:
        sense.set_pixel(3,3,0,0,0)
    if 175 < brightness < 255:
        sense.set_pixel(3,2,255,255,255)
    else:
        sense.set_pixel(3,2,0,0,0)
    if 200 < brightness < 255:
        sense.set_pixel(3,1,255,255,255)
    else:
        sense.set_pixel(3,1,0,0,0)
    if 225 < brightness < 255:
        sense.set_pixel(3,0,255,255,255)
    else:
        sense.set_pixel(3,0,0,0,0)
        
    #Blinking Light#
        
    # this is a visual notification that shows that the program is running
    sense.set_pixel(7,0,0,255,0)
    sleep(0.02)
    sense.set_pixel(7,0,0,0,0)
    sense.set_pixel(6,0,0,255,0)
    sleep(0.02)
    sense.set_pixel(6,0,0,0,0)
    sense.set_pixel(5,0,0,255,0)
    sleep(0.02)
    sense.set_pixel(5,0,0,0,0)
    sense.set_pixel(4,0,0,255,0)
    sleep(0.02)
    sense.set_pixel(4,0,0,0,0)
    sense.set_pixel(3,0,0,255,0)
    sleep(0.02)
    sense.set_pixel(3,0,0,0,0)
    sense.set_pixel(2,0,0,255,0)
    sleep(0.02)
    sense.set_pixel(2,0,0,0,0)
    sense.set_pixel(1,0,0,255,0)
    sleep(0.02)
    sense.set_pixel(1,0,0,0,0)
    sense.set_pixel(0,0,0,255,0)
    sleep(0.02)
    sense.set_pixel(0,0,0,0,0)
    
    ######################################################
    
cap.release()
sense.clear() 
